@@ -64,7 +64,7 @@ static DBusMessage * wpas_dbus_new_success_reply(DBusMessage *message)
 	unsigned int success = 1;
 
 	reply = dbus_message_new_method_return(message);
-	dbus_message_append_args(message, DBUS_TYPE_UINT32, &success,
+	dbus_message_append_args(reply, DBUS_TYPE_UINT32, &success,
 				 DBUS_TYPE_INVALID);
 	return reply;
 }
@@ -938,6 +938,8 @@ DBusMessage * wpas_dbus_iface_set_network(DBusMessage *message,
 					goto error;
 			} else {
 				value = strdup(entry.str_value);
+				if (value == NULL)
+					goto error;
 			}
 		} else if (entry.type == DBUS_TYPE_UINT32) {
 			value = wpa_zalloc(size);
@@ -969,6 +971,7 @@ DBusMessage * wpas_dbus_iface_set_network(DBusMessage *message,
 		continue;
 
 	error:
+		free(value);
 		reply = wpas_dbus_new_invalid_opts_error(message, entry.key);
 		wpa_dbus_dict_entry_clear(&entry);
 		break;
@@ -1110,6 +1113,7 @@ DBusMessage * wpas_dbus_iface_select_network(DBusMessage *message,
 			ssid->disabled = (nid != ssid->id);
 			ssid = ssid->next;
 		}
+		wpa_s->disconnected = 0;
 		wpa_s->reassociate = 1;
 		wpa_supplicant_req_scan(wpa_s, 0, 0);
 	}
@@ -1173,3 +1177,29 @@ DBusMessage * wpas_dbus_iface_set_ap_scan(DBusMessage *message,
 out:
 	return reply;
 }
+
+
+/**
+ * wpas_dbus_iface_get_state - Get interface state
+ * @message: Pointer to incoming dbus message
+ * @wpa_s: wpa_supplicant structure for a network interface
+ * Returns: A dbus message containing a STRING representing the current
+ *          interface state
+ *
+ * Handler function for "state" method call.
+ */
+DBusMessage * wpas_dbus_iface_get_state(DBusMessage *message,
+					struct wpa_supplicant *wpa_s)
+{
+	DBusMessage *reply = NULL;
+	const char *str_state;
+
+	reply = dbus_message_new_method_return(message);
+	if (reply != NULL) {
+		str_state = wpa_supplicant_state_txt(wpa_s->wpa_state);
+		dbus_message_append_args(reply, DBUS_TYPE_STRING, &str_state,
+					 DBUS_TYPE_INVALID);
+	}
+
+	return reply;
+}
